@@ -9,12 +9,17 @@ import json
 from .node import InputNode, OutputNode, ActionNode, Node
 
 
+class ParentNotEvaluated(Exception):
+    pass
+
+
 class ValidFlowPath:
     """Define the progromatic flow, consume list of json described nodes
     and create a series of lists for varrious node types
     """
 
     def __init__(self, node_defs: typing.List[dict]):
+        self.output = []
         self.nodes = {}
         self.input_nodes = []
         self.output_nodes = []
@@ -65,13 +70,17 @@ class ValidFlowPath:
             typing.List[OutputNode] -- List of Output Nodes
         """
         self.track_node_evals = get_node_evals
-        results = []
+        self.output = []
         for i in self.input_nodes:
             for target_id in i.targets:
-                results.extend(self.run_node(target_id, payload))
-        return results
+                self.run_node(target_id, payload)
+        return self.output
 
-    def run_node(self, node_id: str, payload):
+    def run_node(
+        self,
+        node_id: str,
+        payload
+    ) -> typing.List[OutputNode]:
         """Get Node with provided id, return if it's an output node, other wise
         run the node and continue down the path
 
@@ -84,14 +93,15 @@ class ValidFlowPath:
             calls the node's run method
             (which will intern end up returning a OutputNode)
         """
-        node = self.nodes.get(node_id)
+        node: Node = self.nodes.get(node_id)
         if node in self.output_nodes:
-            return node
-        assert isinstance(node, ActionNode)
-        results, node_eval = node.run(self, payload)
+            self.output.append(node)
+            return
+        next_node_ids, node_eval = node.run(payload)
         if self.track_node_evals:
-            self.node_evals.update(node_eval)
-        return results
+            self.node_evals.update({node_id: node_eval})
+        for next_id in next_node_ids:
+            self.run_node(next_id, payload)
 
     @staticmethod
     def vfm_get_path_json(
